from typing import Any, Union, Dict, List, Optional
from uuid import UUID

from satellitevu.apis.base import AbstractApi
from satellitevu.apis.exceptions import IDAPIError


class IdV2(AbstractApi):
    """
    Client interface to the ID API located at
    https://api.satellitevu.com/id/v2/docs
    """

    api_path = "id/v2"

    def get_user_details(self):
        """
        Retrieves the details of a user.

        Returns:
            A dictionary containing properties of the user.
        """
        url = self.url("user/details")
        response = self.make_request(method="GET", url=url)
        return response.json()

    def rotate_client_secret(self):
        """
        Generate a new client secret for the M2M client associated with a user.

        Returns:
            A dictionary containing the client ID and new secret of the user.
        """
        url = self.url("client/reset")
        response = self.make_request(method="POST", url=url)
        return response.json()

    def edit_user_settings(self, notifications: Dict[str, Any]):
        """
        Update user settings.
        """
        url = self.url("user/settings")
        response = self.make_request(
            method="PUT", url=url, json={"notifications": notifications}
        )
        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def get_credit_balance(self, contract_id: Union[UUID, str]):
        """
        Retrieves the credit balance. This is calculated as the credit limit
        for the contract minus the total credits used.
        """
        url = self.url(f"{str(contract_id)}/wallet/credit")
        response = self.make_request(method="GET", url=url)
        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def create_webhook(
        self,
        name: str,
        url: str,
        event_types: List[str],
        **kwargs,
    ):
        """
        Create a webhook.

        Args:
            name: The name of the webhook.

            url: The URL where you want to receive requests for events you are subscribed to.
            Must be HTTPS.

            event_types: A list of events to subscribe to.

        Kwargs:
            Allows sending additional parameters that are supported by the API but not
            added to this SDK yet.

        Returns:
            A dictionary containing properties of the created webhook.
        """
        payload = {
            "event_types": event_types,
            "name": name,
            "url": url,
            **kwargs,
        }

        response = self.make_request(
            method="POST",
            url=self.url("webhooks/"),
            json=payload,
        )

        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def get_webhook(self, webhook_id: Union[UUID, str]):
        """
        Get information about an existing webhook.

        Args:
            webhook_id: The webhook ID.

        Returns:
            A dictionary containing properties of the webhook.
        """
        response = self.make_request(
            method="GET",
            url=self.url(f"webhooks/{str(webhook_id)}/"),
        )
        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def list_webhooks(
        self,
        per_page: int = 25,
        page_token: Optional[str] = None,
    ):
        """
        List all webhooks.

        Args:
            per_page: Number of results (defaults to 25) to be returned per page.

            page_token: Optional string key used to return specific page of results.
            Defaults to None -> assumes page 0.

        Returns:
            A list of dictionaries containing properties of each webhook.
        """

        url = self.url(f"webhooks/?per_page={per_page}")
        if page_token:
            url += f"&token={page_token}"

        response = self.make_request(method="GET", url=url)
        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def edit_webhook(
        self,
        webhook_id: Union[UUID, str],
        active: Optional[bool] = None,
        event_types: Optional[List[str]] = None,
        name: Optional[str] = None,
        **kwargs,
    ):
        """
        Edit a webhook.

        Args:
            webhook_id: The webhook ID.

            active: Whether the webhook should be active or not.

            event_types: A list of events to subscribe to.

            name: The name of the webhook.

        Kwargs:
            Allows sending additional parameters that are supported by the API but not
            added to this SDK yet.

        Returns:
            A dictionary containing properties of the webhook.
        """
        payload = {**kwargs}

        if active:
            payload["active"] = active
        if event_types:
            payload["event_types"] = event_types
        if name:
            payload["name"] = name

        response = self.make_request(
            method="PATCH",
            url=self.url(f"webhooks/{webhook_id}/"),
            json=payload,
        )
        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def delete_webhook(self, webhook_id: Union[UUID, str]):
        """
        Delete a webhook.

        Args:
            webhook_id: The webhook ID.
        """

        response = self.make_request(
            method="DELETE",
            url=self.url(
                f"webhooks/{webhook_id}/",
            ),
        )
        if response.status != 204:
            raise IDAPIError(response.status, response.text)

    def rotate_webhook_signing_key(self, webhook_id: Union[UUID, str]):
        """
        Rotate the signing key for a webhook.

        Args:
            webhook_id: The webhook ID.

        Returns:
            A dictionary containing properties of the webhook,
            including the new signing key.
        """
        response = self.make_request(
            method="POST", url=self.url(f"webhooks/{webhook_id}/rotate/")
        )
        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def test_webhook(self, webhook_id: Union[UUID, str]):
        """
        Test a webhook.

        Args:
            webhook_id: The webhook ID.

        Returns:
            A dictionary containing properties of the webhook
            and details about the result of the test.
        """
        response = self.make_request(
            method="POST", url=self.url(f"/webhooks/{webhook_id}/test/")
        )
        if response.status != 200:
            raise IDAPIError(response.status, response.text)
        return response.json()

    def get_webhook_events(self):
        """
        Retrieves all webhook event types.

        Returns:
            A list of all webhook event types.
        """
        response = self.make_request(method="GET", url=self.url("webhooks/events/"))
        return response.json()



from allure import story, title
from configparser import ConfigParser
from pathlib import Path

from pyfakefs.fake_filesystem import FakeFilesystem

from .cache import AppDirCache, MemoryCache

TEST_DIR = Path("/test")


@story("Cache")
class TestCache:
    @title("Empty cache save")
    def test_empty_cache_save(self, fs: FakeFilesystem):
        fs.makedir(TEST_DIR)
        cache = AppDirCache(TEST_DIR)
        parser = ConfigParser()

        cache.save("test-client", "test-access")

        parser.read(TEST_DIR / "tokencache")
        assert {key: value for key, value in parser.items("test-client")} == {
            "access_token": "test-access",
        }

    @title("Empty cache load")
    def test_empty_cache_load(self, fs: FakeFilesystem):
        fs.makedir(TEST_DIR)
        cache = AppDirCache(TEST_DIR)

        assert cache.load("test-client") is None

    @title("Partial cache save")
    def test_partial_cache_save(self, fs: FakeFilesystem):
        fs.makedir(TEST_DIR)
        cache = AppDirCache(TEST_DIR)
        cache_file = TEST_DIR / "tokencache"

        parser = ConfigParser()
        parser.add_section("other-client")
        parser["other-client"]["access_token"] = "foo"
        with open(cache_file, "w") as handle:
            parser.write(handle)

        cache.save("test-client", "test-access")

        parser = ConfigParser()
        parser.read(cache_file)
        assert {key: value for key, value in parser.items("test-client")} == {
            "access_token": "test-access",
        }
        assert {key: value for key, value in parser.items("other-client")} == {
            "access_token": "foo",
        }

    @title("Cache update")
    def test_cache_update(self, fs: FakeFilesystem):
        fs.makedir(TEST_DIR)
        cache = AppDirCache(TEST_DIR)
        cache_file = TEST_DIR / "tokencache"

        parser = ConfigParser()
        parser.add_section("test-client")
        parser["test-client"]["access_token"] = "foo"
        with open(cache_file, "w") as handle:
            parser.write(handle)

        cache.save("test-client", "bar")

        parser = ConfigParser()
        parser.read(cache_file)
        assert {key: value for key, value in parser.items("test-client")} == {
            "access_token": "bar",
        }

    @title("Empty memory cache")
    def test_memory_cache_empty(self):
        cache = MemoryCache()

        assert cache.load("test-client") is None

    @title("Memory cache")
    def test_memory_cache(self):
        cache = MemoryCache()
        cache.save("test-client", "bar")

        assert cache.load("test-client") == "bar"



from allure import description, story, title
from json import dumps, loads
from itertools import product
from secrets import token_urlsafe
from unittest.mock import patch
from urllib.parse import urlparse
from uuid import uuid4

from mocket import Mocket
from mocket.mockhttp import Entry, Response
from pytest import mark, raises

from satellitevu.apis.exceptions import OTMOrderCancellationError, OTMParametersError

API_PATH_FEASIBILITY = "otm/v2/contract-id/tasking/feasibilities/"
API_PATH_ORDERS = "otm/v2/contract-id/tasking/orders/"


@story("Tasking")
@mark.usefixtures("mocketize_fixture")
class TestTasking:
    @title("Cannot use OTM v2 without contract ID")
    @description("Ensure that OTM v2 methods cannot be used without a contract ID")
    def test_cannot_use_v2_without_contract_id(self, client):
        error_message = "missing 1 required keyword-only argument: 'contract_id'"
        with raises(TypeError) as exc:
            client.otm_v2.get_order(order_id=uuid4())

        assert error_message in str(exc.value)

    @title("Feasibility POST request")
    @description("Test the POST request for feasibility")
    @mark.parametrize("product", ("standard", "assured"))
    def test_post_feasibility(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_response,
        product,
    ):
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            body=dumps(otm_response),
            status=202,
        )

        response = client.otm_v2.post_feasibility(
            **otm_request_parameters,
            product=product,
        )
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        api_request_body = loads(api_request.body)
        assert api_request_body["geometry"] == {
            "type": "Point",
            "coordinates": otm_request_parameters["coordinates"],
        }

        properties_keys = ["max_cloud_cover", "min_off_nadir", "max_off_nadir"]

        if product == "assured":
            for key in properties_keys:
                assert not api_request_body["properties"].get(key)
        elif product == "standard":
            for key in properties_keys:
                assert (
                    api_request_body["properties"][key] == otm_request_parameters[key]
                )

        assert (
            api_request_body["properties"]["datetime"]
            == f"{otm_request_parameters['date_from'].isoformat()}/{otm_request_parameters['date_to'].isoformat()}"  # noqa: E501
        )

    @title("Create a feasibility request")
    @description("Test the POST request for feasibility with off-nadir and GSD values")
    @mark.parametrize("product", ("standard", "assured"))
    def test_post_feasibility_off_nadir_gsd_values(
        self, oauth_token_entry, client, otm_request_parameters, product, otm_response
    ):
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))

        otm_request_parameters["product"] = product
        for param in ["min_off_nadir", "max_off_nadir", "min_gsd", "max_gsd"]:
            otm_request_parameters[param] = None

        if product == "assured":
            Entry.single_register(
                "POST",
                client._gateway_url + api_path,
                body=dumps(otm_response),
                status=202,
            )

            response = client.otm_v2.post_feasibility(
                **otm_request_parameters,
            )
            assert isinstance(response, dict)

        else:
            with raises(OTMParametersError):
                client.otm_v2.post_feasibility(**otm_request_parameters)

    @title("Feasibility GET request")
    @description("Get details about a feasibility request")
    @mark.parametrize("pact", ["otm"], indirect=True)
    def test_get_feasibility(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_feasibility_body,
        pact,
    ):
        feasibility_id = uuid4()
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))

        Entry.single_register(
            "GET",
            client._gateway_url + api_path + str(feasibility_id),
            status=200,
            body=dumps(otm_feasibility_body),
        )

        response = client.otm_v2.get_feasibility(
            contract_id=contract_id, id=feasibility_id
        )

        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path + f"{feasibility_id}"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Retrieve details of a feasibility request")
            .given("the feasibility request exists")
            .with_request(
                method="GET",
                path=f"/{contract_id}/tasking/feasibilities/{feasibility_id}",
            )
            .will_respond_with(200)
            .with_body(otm_feasibility_body)
        )

    @title("Get feasibility response")
    @description("Test the GET request for feasibility response")
    @mark.parametrize("pact", ["otm"], indirect=True)
    def test_get_feasibility_response(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_feasibility_response_body,
        pact,
    ):
        feasibility_id = uuid4()
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))

        Entry.single_register(
            "GET",
            client._gateway_url + api_path + str(feasibility_id) + "/response",
            status=200,
            body=dumps(otm_feasibility_response_body),
        )

        response = client.otm_v2.get_feasibility_response(
            contract_id=contract_id, id=feasibility_id
        )

        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path + f"{feasibility_id}/response"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Retrieve details of a feasibility request response")
            .given("the feasibility request exists")
            .with_request(
                method="GET",
                path=f"/{contract_id}/tasking/feasibilities/{feasibility_id}/response",
            )
            .will_respond_with(200)
            .with_body(otm_feasibility_response_body)
        )

    @title("List feasibilities")
    @description("Test the GET request for listing feasibilities")
    @mark.parametrize(
        "per_page",
        (None, 10),
    )
    def test_list_feasibilities(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_response,
        per_page,
    ):
        contract_id = otm_request_parameters["contract_id"]
        per_page = per_page if not None else 25
        api_path = (
            API_PATH_FEASIBILITY.replace("contract-id", str(contract_id))
            + f"?{per_page=}"
        )

        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=200,
            body=dumps(otm_response),
        )

        response = client.otm_v2.list_feasibility_requests(
            contract_id=contract_id, per_page=per_page
        )

        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

    @title("Create a tasking order")
    @description("Test the POST request for creating a tasking order")
    @mark.parametrize("product", ("standard", "assured"))
    @mark.parametrize("pact", ["otm"], indirect=True)
    def test_post_order(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_order_confirmation,
        product,
        pact,
    ):
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))

        if product == "assured":
            otm_request_parameters["product"] = product
            otm_request_parameters["signature"] = token_urlsafe(16)

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            body=dumps(otm_order_confirmation),
            status=201,
        )

        response = client.otm_v2.create_order(**otm_request_parameters)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        api_request_body = loads(api_request.body)
        print(api_request_body)
        properties_keys = ["max_cloud_cover", "min_off_nadir", "max_off_nadir"]

        if product == "assured":
            assert "geometry" not in api_request_body.keys()
            assert "signature" in api_request_body["properties"].keys()
            for key in properties_keys:
                assert not api_request_body["properties"].get(key)
        else:
            assert api_request_body["geometry"] == {
                "type": "Point",
                "coordinates": otm_request_parameters["coordinates"],
            }
            assert "signature" not in api_request_body["properties"].keys()
            for key in properties_keys:
                assert (
                    api_request_body["properties"][key] == otm_request_parameters[key]
                )
            assert (
                api_request_body["properties"]["datetime"]
                == f"{otm_request_parameters['date_from'].isoformat()}/{otm_request_parameters['date_to'].isoformat()}"  # noqa: E501
            )

            (
                pact.upon_receiving(f"A tasking order ({product})")
                .given("contract has credit")
                .with_request(method="POST", path=f"/{contract_id}/tasking/orders")
                .with_body(api_request_body)
                .will_respond_with(201)
                .with_body(otm_order_confirmation)
            )

    @title("Create a tasking order")
    @mark.parametrize("product", ("standard", "assured"))
    def test_create_order_off_nadir_gsd_values(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        product,
        otm_response,
    ):
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))

        otm_request_parameters["product"] = product
        otm_request_parameters["signature"] = token_urlsafe(16)
        for param in ["min_off_nadir", "max_off_nadir", "min_gsd", "max_gsd"]:
            otm_request_parameters[param] = None

        if product == "assured":
            Entry.single_register(
                "POST",
                client._gateway_url + api_path,
                body=dumps(otm_response),
                status=201,
            )

            response = client.otm_v2.create_order(**otm_request_parameters)
            assert isinstance(response, dict)
        else:
            with raises(OTMParametersError):
                client.otm_v2.create_order(**otm_request_parameters)

    @title("Create a tasking order with addons")
    @description("Test the POST request for creating a tasking order with addons")
    @mark.parametrize(
        "product, withhold",
        product(
            ("standard", "assured"),
            (
                "0d",
                "7d",
                "1y",
            ),
        ),
    )
    @mark.parametrize("pact", ["otm"], indirect=True)
    def test_create_order_with_addons(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        product,
        withhold,
        otm_order_confirmation,
        pact,
    ):
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))

        otm_request_parameters["product"] = product
        otm_request_parameters["signature"] = token_urlsafe(16)
        otm_request_parameters["addon_withhold"] = withhold

        if product == "assured":
            Entry.single_register(
                "POST",
                client._gateway_url + api_path,
                body=dumps(otm_order_confirmation),
                status=201,
            )

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            body=dumps(otm_order_confirmation),
            status=201,
        )

        response = client.otm_v2.create_order(**otm_request_parameters)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        api_request_body = loads(api_request.body)

        assert api_request_body["properties"]["product"] == product

        if withhold:
            assert api_request_body["properties"]["addon:withhold"] == withhold
        else:
            assert "addon:withhold" not in api_request_body["properties"].keys()

        (
            pact.upon_receiving(f"A tasking order with addons ({product})")
            .given("contract has credit")
            .with_request(method="POST", path=f"/{contract_id}/tasking/orders")
            .with_body(api_request_body)
            .will_respond_with(201)
            .with_body(otm_order_confirmation)
        )

    @title("Create a tasking order without signature")
    @description(
        "Test the POST request for creating a tasking order without a signature"
    )
    def test_post_assured_order_without_signature(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_response,
    ):
        otm_request_parameters["product"] = "assured"

        with raises(
            Exception,
            match="Orders with assured priority must have a signature token.",
        ):
            client.otm_v2.create_order(**otm_request_parameters)

    @title("List tasking orders")
    @description("Retrieve a list of tasking orders")
    @mark.parametrize(
        "per_page",
        (None, 10),
    )
    def test_list_tasking_orders(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_response,
        per_page,
    ):
        contract_id = otm_request_parameters["contract_id"]
        per_page = per_page if not None else 25
        api_path = (
            API_PATH_ORDERS.replace("contract-id", str(contract_id)) + f"?{per_page=}"
        )

        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=200,
            body=dumps(otm_response),
        )

        response = client.otm_v2.list_orders(contract_id=contract_id, per_page=per_page)

        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

    @title("Get tasking order")
    @description("Retrieve a specific tasking order")
    @mark.parametrize("pact", ["otm"], indirect=True)
    def test_get_tasking_order(
        self,
        oauth_token_entry,
        client,
        otm_request_parameters,
        otm_order_response,
        pact,
    ):
        order_id = uuid4()
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))

        Entry.single_register(
            "GET",
            client._gateway_url + api_path + str(order_id),
            status=200,
            body=dumps(otm_order_response),
        )

        response = client.otm_v2.get_order(contract_id=contract_id, order_id=order_id)

        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path + f"{order_id}"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Retrieve details of a tasking order")
            .given("the order exists")
            .with_request(
                method="GET", path=f"/{contract_id}/tasking/orders/{order_id}"
            )
            .will_respond_with(200)
            .with_body(otm_order_response)
        )

    @title("Cancel tasking order")
    @description("Cancel a specific tasking order")
    @mark.parametrize("status_code", (204, 404, 409))
    @mark.parametrize("pact", ["otm"], indirect=True)
    def test_cancel_tasking_order(
        self, oauth_token_entry, client, otm_request_parameters, status_code, pact
    ):
        order_id = uuid4()
        contract_id = otm_request_parameters["contract_id"]
        api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))

        Entry.single_register(
            "POST",
            client._gateway_url + api_path + str(order_id) + "/cancel",
            status=status_code,
        )

        if status_code == 204:
            response = client.otm_v2.cancel_order(
                contract_id=contract_id, order_id=order_id
            )
            assert not response
        else:
            with raises(OTMOrderCancellationError):
                client.otm_v2.cancel_order(contract_id=contract_id, order_id=order_id)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path + f"{order_id}" + "/cancel"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Cancel a tasking order")
            .given("the order exists")
            .with_request(
                method="POST", path=f"/{contract_id}/tasking/orders/{order_id}/cancel"
            )
            .will_respond_with(204)
        )

    @title("Search for tasking orders")
    @description("Search for tasking orders")
    def test_post_search(
        self, oauth_token_entry, client, otm_request_parameters, search_response
    ):
        contract_id = otm_request_parameters["contract_id"]
        api_path = "otm/v2/contract-id/search/".replace("contract-id", str(contract_id))

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            status=200,
            body=dumps(search_response),
        )

        search_parameters = {
            "per_page": 100,
            "collections": ["feasibility"],
            "date_range": "2023-01-01T00:00:00/2023-01-31T11:59:59",
            "properties": {"min_gsd": 15, "max_gsd": 35, "status": "feasible"},
            "intersects": {},
            "sort_by": [{"field": "min_gsd", "direction": "asc"}],
        }

        response = client.otm_v2.search(contract_id=contract_id, **search_parameters)

        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

    @title("Download tasking order")
    @description("Download a specific tasking order")
    def test_download_order(
        self,
        client,
        oauth_token_entry,
        redirect_response,
    ):
        contract_id = str(uuid4())
        api_path = API_PATH_ORDERS.replace("contract-id", str(contract_id))
        order_id = "528b0f77-5df1-4ed7-9224-502817170613"
        download_dir = "downloads"

        Entry.register(
            "GET",
            client._gateway_url + f"{api_path}{order_id}/download?redirect=False",
            Response(headers={"Retry-After": "1"}, status=202),
            Response(body=dumps(redirect_response), status=200),
        )
        Entry.single_register("GET", uri=redirect_response["url"])

        with patch("satellitevu.apis.otm.bytes_to_file") as mock_file_dl:
            mock_file_dl.return_value = f"{download_dir}/{order_id}.zip"

            response = client.otm_v2.download_order(
                contract_id=contract_id, order_id=order_id, destdir=download_dir
            )

        requests = Mocket.request_list()

        assert len(requests) == 4

        api_request = requests[1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}{order_id}/download?redirect=False"
        assert api_request.headers["authorization"] == oauth_token_entry

        mock_file_dl.assert_called_once()
        assert response == mock_file_dl()
        assert isinstance(response, str)

        Mocket.assert_fail_if_entries_not_served()



import allure
from calendar import timegm
from datetime import datetime, timedelta, timezone
from importlib import import_module
from json import dumps
from typing import Dict, Optional, Callable
from urllib.parse import urljoin
from uuid import uuid4
from collections.abc import Generator
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.serialization import (
    Encoding,
    PrivateFormat,
    NoEncryption,
)
from cryptography.hazmat.primitives.asymmetric.rsa import generate_private_key
from josepy import JWKRSA
from jwt import PyJWK, encode
from mocket import mocketize, Mocketizer
from mocket.mockhttp import Entry
from pact.v3 import Pact
from pytest import fixture, mark, param

try:
    from requests import Session as RequestsSession
except ImportError:
    RequestsSession = None
try:
    from httpx import Client as HttpxClient
except ImportError:
    HttpxClient = None

from satellitevu.auth.cache import AbstractCache
from satellitevu.client import Client


@fixture(
    params=(
        param("UrllibClient"),
        param(
            "requests.RequestsSession",
            marks=[
                mark.skipif(RequestsSession is None, reason="requests is not installed")
            ],
        ),
        param(
            "httpx.HttpxClient",
            marks=[mark.skipif(HttpxClient is None, reason="httpx is not installed")],
        ),
    )
)
def http_client_class(request):
    full_path = f"satellitevu.http.{request.param}"
    module = import_module(full_path.rsplit(".", maxsplit=1)[0])
    return getattr(module, full_path.rsplit(".")[-1])


class MemoryCache(AbstractCache):
    cache: Dict[str, str]

    def __init__(self):
        super().__init__()
        self.cache = {}

    def save(self, client_id: str, value: str):
        self.cache[client_id] = value

    def load(self, client_id: str) -> Optional[str]:
        return self.cache.get(client_id)


@fixture
def client(memory_cache):
    return Client(client_id="mock-id", client_secret="mock-secret", cache=memory_cache)


@fixture
def memory_cache():
    return MemoryCache()


@fixture
def oauth_token_entry(client, auth0_token_factory) -> str:
    token = auth0_token_factory("John Doe")
    Entry.single_register(
        "POST",
        urljoin(client.auth.auth_url, "oauth/token"),
        body=dumps({"access_token": token}),
    )
    return f"Bearer {token}"


@fixture(scope="session")
def jwk():
    key = generate_private_key(
        backend=default_backend,
        public_exponent=65537,
        key_size=1024,
    ).private_bytes(
        Encoding.PEM,
        PrivateFormat.PKCS8,
        NoEncryption(),
    )
    return JWKRSA.load(key).to_json()


@fixture(scope="session")
def auth_url() -> str:
    return "http://auth.example.com"


@mocketize(strict_mode=True)
@fixture(scope="function")
def generic_token_factory(
    jwk,
    auth_url,
) -> Callable[..., str]:
    def factory(ttl=300, **claims):
        """
        Generate JWT for given claims with a given TTL, signed with the provider's RSA
        key.
        """
        now = timegm(datetime.now(tz=timezone.utc).utctimetuple())
        payload = {
            **claims,
            "iat": now,
            "exp": now + ttl,
        }
        jwk_obj = PyJWK(jwk, algorithm="RS256")
        return encode(payload, jwk_obj.key, algorithm=jwk_obj.algorithm_name)

    return factory


@fixture(scope="function")
def auth0_token_factory(generic_token_factory):
    def factory(name: str, *, ttl=300, **access_claims):
        given, family = name.split(" ")
        return generic_token_factory(
            ttl=ttl,
            iss=access_claims.pop("iss", "test"),
            azp="test",
            sub=f"{given.lower()}_{family.lower()}",
            scope=access_claims.pop("scope", "openid"),
            **access_claims,
        )

    return factory


@fixture
def order_list_response():
    return {
        "orders": [
            {
                "id": "06b45f92-f574-43e1-88ea-0725a99f69c6",
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "id": "3749aa20-7a0a-4322-823d-b27aa5981eaf",
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": [
                                [
                                    [23.381329637, 29.62479696],
                                    [23.324715343, 29.623895588],
                                    [23.327505896, 29.587107204],
                                    [23.384102168, 29.588120601],
                                    [23.381329637, 29.62479696],
                                ]
                            ],
                        },
                        "properties": {
                            "item_id": "20231208T115432000_visual_30_hotsat1",
                            "order_id": "06b45f92-f574-43e1-88ea-0725a99f69c6",
                            "created_at": "2024-10-18T11:05:35Z",
                            "stac_metadata": {
                                "id": "20231208T115432000_visual_30_hotsat1",
                                "collection": "visual",
                                "assets": {
                                    "thumbnail": {
                                        "href": "https://images.test.satvu.com/stac-ingestion/hotsat1/20231208T115432000_visual_30_hotsat1_thumbnail.png",
                                        "type": "image/png",
                                        "roles": ["thumbnail"],
                                    }
                                },
                                "bbox": [
                                    23.323807253417414,
                                    29.586046830070046,
                                    23.38498589650168,
                                    29.625820737625705,
                                ],
                                "properties": {
                                    "eo:cloud_cover": 0.0,
                                    "datetime": "2023-12-08T11:54:32Z",
                                    "gsd": 4.142945289611816,
                                    "platform": "hotsat-1",
                                    "view:azimuth": 284.16017649457757,
                                    "view:off_nadir": 21.334559545721035,
                                    "view:sun_azimuth": 206.56290529852637,
                                    "view:sun_elevation": 32.819692975864264,
                                },
                            },
                            "price": {"value": 2, "currency": "GBP"},
                        },
                    },
                    {
                        "type": "Feature",
                        "id": "b16989e5-90c4-4f5f-bbad-cb5c1a0c21a6",
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": [
                                [
                                    [17.094034765, -22.534767225],
                                    [17.09845738, -22.51733564],
                                    [17.102228197, -22.497986319],
                                    [17.046960337, -22.489298278],
                                    [17.042233475, -22.509005634],
                                    [17.039040857, -22.525958237],
                                    [17.094034765, -22.534767225],
                                ]
                            ],
                        },
                        "properties": {
                            "item_id": "20231208T120814000_visual_30_hotsat1",
                            "order_id": "06b45f92-f574-43e1-88ea-0725a99f69c6",
                            "created_at": "2024-10-18T11:05:35Z",
                            "stac_metadata": {
                                "id": "20231208T120814000_visual_30_hotsat1",
                                "collection": "visual",
                                "assets": {
                                    "thumbnail": {
                                        "href": "https://images.test.satvu.com/stac-ingestion/hotsat1/20231208T120814000_visual_30_hotsat1_thumbnail.png",
                                        "type": "image/png",
                                        "roles": ["thumbnail"],
                                    }
                                },
                                "bbox": [
                                    17.038442295262946,
                                    -22.53553050270601,
                                    17.102804643698693,
                                    -22.488568972304773,
                                ],
                                "properties": {
                                    "eo:cloud_cover": 0.0,
                                    "datetime": "2023-12-08T12:08:14Z",
                                    "gsd": 4.266286373138428,
                                    "platform": "hotsat-1",
                                    "view:azimuth": 91.71971393015937,
                                    "view:off_nadir": 24.376613013992234,
                                    "view:sun_azimuth": 265.30216497291434,
                                    "view:sun_elevation": 70.46137872028515,
                                },
                            },
                            "price": {"value": 2, "currency": "GBP"},
                        },
                    },
                ],
                "owned_by": "string",
                "created_at": "2024-10-18T11:05:35Z",
                "updated_at": "2024-10-18T11:05:35Z",
                "contract_id": "4de7e9f1-dcf1-484a-8554-a8f6bb7c22e2",
                "price": {"value": 2, "currency": "GBP"},
            },
            {
                "id": "41de67e1-cb62-45dc-aa94-b026aff9f776",
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "id": "ed2cc9c5-625a-4fbc-8ac6-67465198e57a",
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": [
                                [
                                    [51.322892628, 25.145864273],
                                    [51.304924939, 25.15127406],
                                    [51.28676628, 25.115093129],
                                    [51.345897132, 25.095998054],
                                    [51.364255123, 25.132339095],
                                    [51.322892628, 25.145864273],
                                ]
                            ],
                        },
                        "properties": {
                            "item_id": "20231208T102026000_visual_30_hotsat1",
                            "order_id": "41de67e1-cb62-45dc-aa94-b026aff9f776",
                            "created_at": "2024-10-04T09:56:39Z",
                            "stac_metadata": {
                                "id": "20231208T102026000_visual_30_hotsat1",
                                "collection": "visual",
                                "assets": {
                                    "thumbnail": {
                                        "href": "https://images.test.satvu.com/stac-ingestion/hotsat1/20231208T102026000_visual_30_hotsat1_thumbnail.png",
                                        "type": "image/png",
                                        "roles": ["thumbnail"],
                                    }
                                },
                                "bbox": [
                                    51.286698951221425,
                                    25.09593322503868,
                                    51.364380138663044,
                                    25.15133119478952,
                                ],
                                "properties": {
                                    "eo:cloud_cover": 0.0,
                                    "datetime": "2023-12-08T10:20:26Z",
                                    "gsd": 4.647026062011719,
                                    "platform": "hotsat-1",
                                    "view:azimuth": 95.02176620678249,
                                    "view:off_nadir": 29.182734049482512,
                                    "view:sun_azimuth": 212.39816698394057,
                                    "view:sun_elevation": 34.77974478735089,
                                },
                            },
                            "price": {"value": 3, "currency": "GBP"},
                        },
                    },
                    {
                        "type": "Feature",
                        "id": "f983ebb8-e02d-4960-bcfc-e53df611ea74",
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": [
                                [
                                    [33.55354294, 44.591551499],
                                    [33.561941514, 44.60597847],
                                    [33.571026516, 44.625606529],
                                    [33.495996855, 44.643580806],
                                    [33.477326556, 44.607060213],
                                    [33.551625793, 44.588978666],
                                    [33.55354294, 44.591551499],
                                ]
                            ],
                        },
                        "properties": {
                            "item_id": "20231208T115018000_visual_30_hotsat1",
                            "order_id": "41de67e1-cb62-45dc-aa94-b026aff9f776",
                            "created_at": "2024-10-04T09:56:39Z",
                            "stac_metadata": {
                                "id": "20231208T115018000_visual_30_hotsat1",
                                "collection": "visual",
                                "assets": {
                                    "thumbnail": {
                                        "href": "https://images.test.satvu.com/20231208T115018000_visual_30_hotsat1_thumbnail.png",
                                        "type": "image/png",
                                        "roles": ["thumbnail"],
                                    }
                                },
                                "bbox": [
                                    33.47715177281742,
                                    44.588782508565636,
                                    33.57122826986277,
                                    44.64367972113274,
                                ],
                                "properties": {
                                    "eo:cloud_cover": 0.0,
                                    "datetime": "2023-12-08T11:50:18Z",
                                    "gsd": 4.552684307098389,
                                    "platform": "hotsat-1",
                                    "view:azimuth": 105.29726092466655,
                                    "view:off_nadir": 28.41357725065772,
                                    "view:sun_azimuth": 211.6914436210348,
                                    "view:sun_elevation": 16.22401010212522,
                                },
                            },
                            "price": {"value": 3, "currency": "GBP"},
                        },
                    },
                    {
                        "type": "Feature",
                        "id": "416481a9-f158-4363-b637-dd8c28ca2cb1",
                        "geometry": {
                            "type": "Polygon",
                            "coordinates": [
                                [
                                    [23.381329637, 29.62479696],
                                    [23.324715343, 29.623895588],
                                    [23.327505896, 29.587107204],
                                    [23.384102168, 29.588120601],
                                    [23.381329637, 29.62479696],
                                ]
                            ],
                        },
                        "properties": {
                            "item_id": "20231208T115432000_visual_30_hotsat1",
                            "order_id": "41de67e1-cb62-45dc-aa94-b026aff9f776",
                            "created_at": "2024-10-04T09:56:39Z",
                            "stac_metadata": {
                                "id": "20231208T115432000_visual_30_hotsat1",
                                "collection": "visual",
                                "assets": {
                                    "thumbnail": {
                                        "href": "https://images.test.satvu.com/stac-ingestion/hotsat1/20231208T115432000_visual_30_hotsat1_thumbnail.png",
                                        "type": "image/png",
                                        "roles": ["thumbnail"],
                                    }
                                },
                                "bbox": [
                                    23.323807253417414,
                                    29.586046830070046,
                                    23.38498589650168,
                                    29.625820737625705,
                                ],
                                "properties": {
                                    "eo:cloud_cover": 0.0,
                                    "datetime": "2023-12-08T11:54:32Z",
                                    "gsd": 4.142945289611816,
                                    "platform": "hotsat-1",
                                    "view:azimuth": 284.16017649457757,
                                    "view:off_nadir": 21.334559545721035,
                                    "view:sun_azimuth": 206.56290529852637,
                                    "view:sun_elevation": 32.819692975864264,
                                },
                            },
                            "price": {"value": 3, "currency": "GBP"},
                        },
                    },
                ],
                "owned_by": "string",
                "created_at": "2024-10-04T09:56:39Z",
                "updated_at": "2024-10-04T09:56:39Z",
                "contract_id": "4de7e9f1-dcf1-484a-8554-a8f6bb7c22e2",
                "price": {"value": 3, "currency": "GBP"},
            },
        ],
        "links": [],
    }


@fixture
def redirect_response():
    return {"url": "https://image.test", "ttl": 3600}


@fixture
def order_details_response():
    return {
        "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
                "geometry": {"type": "Point", "coordinates": [0, 0]},
                "properties": {
                    "item_id": "20231110T173102000_visual_30_hotsat",
                    "order_id": "1cfdc8cc-968d-406c-bada-d45c55a26857",
                    "created_at": "2019-08-24T14:15:22Z",
                    "stac_metadata": {
                        "id": "string",
                        "collection": "string",
                        "assets": {
                            "property1": {
                                "href": "http://example.com",
                                "type": "string",
                                "roles": ["string"],
                            },
                            "property2": {
                                "href": "http://example.com",
                                "type": "string",
                                "roles": ["string"],
                            },
                        },
                        "bbox": [0, 0, 0, 0],
                        "properties": {
                            "eo:cloud_cover": 0,
                            "datetime": "2019-08-24T14:15:22Z",
                            "gsd": 0,
                            "platform": "string",
                            "view:azimuth": 0,
                            "view:off_nadir": 0,
                            "view:sun_azimuth": 0,
                            "view:sun_elevation": 0,
                        },
                    },
                    "price": {"value": 100000, "currency": "GBP"},
                },
            }
        ],
        "owned_by": "John Doe",
        "created_at": "2019-08-24T14:15:22Z",
        "updated_at": "2019-08-24T14:15:22Z",
        "contract_id": "9aafc1a8-e497-46c9-ba0b-bd5b03c353e4",
        "price": {"value": 100000, "currency": "GBP"},
    }


@fixture
def otm_request_parameters(withhold="0d"):
    now = datetime.now(timezone.utc)
    return {
        "addon_withhold": withhold,
        "coordinates": [0, 0],
        "date_from": now,
        "date_to": (now + timedelta(hours=24)),
        "day_night_mode": "night",
        "max_cloud_cover": 100,
        "min_off_nadir": 20,
        "max_off_nadir": 30,
        "contract_id": str(uuid4()),
    }


@fixture
def otm_response(otm_request_parameters):
    return {
        "bbox": [0.0, 0.0, 0.0, 0.0],
        "type": "Feature",
        "geometry": {
            "type": "Point",
            "coordinates": otm_request_parameters["coordinates"],
        },
        "properties": {
            "datetime": (
                f"{otm_request_parameters['date_from']}/"
                f"{otm_request_parameters['date_to']}"
            ),
            "satvu:day_night_mode": otm_request_parameters["day_night_mode"],
            "max_cloud_cover": otm_request_parameters["max_cloud_cover"],
            "min_off_nadir": otm_request_parameters["min_off_nadir"],
            "max_off_nadir": otm_request_parameters["min_off_nadir"],
            "min_gsd": 3.5,
            "max_gsd": 6.8,
            "status": "feasible",
        },
        "id": str(uuid4()),
        "contract_id": str(otm_request_parameters["contract_id"]),
        "links": [],
    }


@fixture
def withhold(request):
    return request.param


@fixture
def otm_order_confirmation(otm_request_parameters):
    return {
        "type": "Feature",
        "geometry": {
            "bbox": [0, 0, 0, 0],
            "type": "Point",
            "coordinates": otm_request_parameters["coordinates"],
        },
        "properties": {
            "addon:withhold": otm_request_parameters["addon_withhold"],
            "product": "standard",
            "datetime": (
                f"{otm_request_parameters['date_from']}/"
                f"{otm_request_parameters['date_to']}"
            ),
            "satvu:day_night_mode": "day",
            "max_cloud_cover": otm_request_parameters["max_cloud_cover"],
            "min_off_nadir": otm_request_parameters["min_off_nadir"],
            "max_off_nadir": otm_request_parameters["min_off_nadir"],
            "min_gsd": 3.5,
            "max_gsd": 6.8,
        },
    }


@fixture
def otm_order_response(otm_request_parameters):
    return {
        "type": "Feature",
        "geometry": {"bbox": [0, 0, 0, 0], "type": "Point", "coordinates": [0, 0]},
        "properties": {
            "addon:withhold": "0d",
            "product": "standard",
            "datetime": "2023-03-22T12:50:24+01:00/2023-03-23T12:50:24+01:00",
            "satvu:day_night_mode": "day",
            "max_cloud_cover": 15,
            "min_off_nadir": 0,
            "max_off_nadir": 30,
            "min_gsd": 3.5,
            "max_gsd": 6.8,
            "status": "committed",
            "created_at": "2019-08-24T14:15:22Z",
            "updated_at": "2019-08-24T14:15:22Z",
        },
        "id": "1ed44bb9-064e-4973-b15a-c5cc460d6677",
        "links": [
            {
                "href": "http://example.com",
                "rel": "string",
                "method": "GET",
                "body": {},
                "merge": False,
                "type": "string",
                "title": "string",
            }
        ],
        "contract_id": otm_request_parameters["contract_id"],
        "price": {
            "currency": "GBP",
            "base": 100000,
            "addon:withhold": 10000,
            "total": 110000,
            "value": 110000,
        },
    }


@fixture()
def otm_feasibility_body(otm_request_parameters):
    return {
        "bbox": [0, 0, 0, 0],
        "type": "Feature",
        "geometry": {"bbox": [0, 0, 0, 0], "type": "Point", "coordinates": [0, 0]},
        "properties": {
            "product": "standard",
            "datetime": "2023-03-22T12:50:24+01:00/2023-03-23T12:50:24+01:00",
            "satvu:day_night_mode": "day",
            "max_cloud_cover": 15,
            "min_off_nadir": 0,
            "max_off_nadir": 30,
            "min_gsd": 3.5,
            "max_gsd": 6.8,
            "status": "pending",
            "created_at": "2019-08-24T14:15:22Z",
            "updated_at": "2019-08-24T14:15:22Z",
            "price": {"value": 100000, "currency": "GBP"},
        },
        "id": "1ed44bb9-064e-4973-b15a-c5cc460d6677",
        "links": [
            {
                "href": "http://example.com",
                "rel": "string",
                "method": "GET",
                "body": {},
                "merge": False,
                "type": "string",
                "title": "string",
            }
        ],
        "contract_id": otm_request_parameters["contract_id"],
    }


@fixture
def otm_feasibility_response_body(otm_request_parameters):
    return {
        "bbox": [0, 0, 0, 0],
        "type": "FeatureCollection",
        "features": [
            {
                "bbox": [0, 0, 0, 0],
                "type": "Feature",
                "geometry": {
                    "bbox": [0, 0, 0, 0],
                    "type": "Point",
                    "coordinates": [0, 0],
                },
                "properties": {
                    "product": "standard",
                    "datetime": "2023-03-22T12:50:24+01:00/2023-03-23T12:50:24+01:00",
                    "satvu:day_night_mode": "day",
                    "max_cloud_cover": 15,
                    "min_off_nadir": 0,
                    "max_off_nadir": 30,
                    "min_gsd": 3.5,
                    "max_gsd": 6.8,
                    "created_at": "2019-08-24T14:15:22Z",
                    "updated_at": "2019-08-24T14:15:22Z",
                    "price": {"value": 100000, "currency": "GBP"},
                    "min_sun_el": 0,
                    "max_sun_el": 0,
                },
                "id": "1ed44bb9-064e-4973-b15a-c5cc460d6677",
            }
        ],
        "id": "1ed44bb9-064e-4973-b15a-c5cc460d6677",
        "links": [
            {
                "href": "http://example.com",
                "rel": "string",
                "method": "GET",
                "body": {},
                "merge": False,
                "type": "string",
                "title": "string",
            }
        ],
        "status": "pending",
        "contract_id": otm_request_parameters["contract_id"],
    }


@fixture
def search_response(otm_request_parameters):
    return {
        "type": "FeatureCollection",
        "features": [
            {
                "type": "Feature",
                "geometry": {
                    "type": "Point",
                    "coordinates": otm_request_parameters["coordinates"],
                },
                "properties": {
                    "datetime": (
                        f"{otm_request_parameters['date_from']}/"
                        f"{otm_request_parameters['date_to']}"
                    ),
                    "status": "feasible",
                    "max_gsd": 6.8,
                    "min_gsd": 3.5,
                    "max_off_nadir": 45,
                    "min_off_nadir": 0,
                    "day_night_mode": "day",
                    "max_cloud_cover": None,
                    "created_at": "2023-10-06T08:56:26Z",
                    "updated_at": "2023-10-06T08:56:28Z",
                },
                "id": "57645c80-fda4-4116-9270-6f1b2a097000",
                "contract_id": "74841d4e-60d4-4e73-a691-2dddffa10d92",
                "collection": "feasibility",
            },
        ],
        "context": {"limit": 25, "matched": 1, "returned": 1},
        "links": [],
    }


@fixture(autouse=True)
def mocketize_fixture():
    with Mocketizer():
        yield


@fixture(scope="session", autouse=True)
def _setup_pact_logging() -> None:
    """
    Set up logging for the pact package.
    """
    from pact.v3 import ffi

    ffi.log_to_stderr("INFO")


@fixture
def pact(request) -> Generator[Pact, None, None]:
    pact_dir = "./pacts"
    pact = Pact("python-sdk", request.param)
    yield pact.with_specification("V4")
    pact.write_file(pact_dir)


@fixture(autouse=True)
def allure_metadata():
    """Allure labels to be added at runtime"""
    allure.dynamic.label("layer", "unit")
    allure.dynamic.feature("Python SDK")



from allure import description, story, title
import json
import re
from urllib.parse import urlparse
from uuid import uuid4

from pytest import raises, mark
from mocket import Mocket
from mocket.mockhttp import Entry

from satellitevu.apis.exceptions import IDAPIError


@mark.parametrize("pact", ["id-service"], indirect=True)
@mark.usefixtures("mocketize_fixture")
@story("ID")
class TestID:
    @title("Get user details")
    @description("Retrieve the details of a user.")
    def test_get_user_details(self, oauth_token_entry, client, pact):
        api_path = "id/v2/user/details"

        id_response_body = json.dumps(
            {
                "user_id": "string",
                "name": "string",
                "email": "string",
                "user_metadata": {},
                "last_login": "string",
            }
        )

        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=200,
            body=id_response_body,
        )

        response = client.id_v2.get_user_details()

        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Request for user information")
            .given("user exists")
            .with_request(method="GET", path="/user/details")
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Rotate client secret")
    @description("Rotate the client secret.")
    def test_rotate_client_secret(self, oauth_token_entry, client, pact):
        api_path = "id/v2/client/reset"
        id_response_body = json.dumps(
            {
                "client_id": "client_id",
                "client_secret": "client_secret",  # pragma: allowlist secret
            }
        )

        Entry.single_register(
            "POST", client._gateway_url + api_path, status=200, body=id_response_body
        )

        response = client.id_v2.rotate_client_secret()
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Request to rotate the client secret")
            .given("a client exists")
            .with_request(method="POST", path="/client/reset")
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Edit user settings")
    @description("Edit the user settings.")
    def test_edit_user_settings(self, oauth_token_entry, client, pact):
        api_path = "id/v2/user/settings"
        id_response_body = json.dumps(
            {
                "user_id": "string",
                "name": "string",
                "email": "string",
                "user_metadata": {},
                "last_login": "string",
            }
        )
        Entry.single_register(
            "PUT", client._gateway_url + api_path, status=200, body=id_response_body
        )
        notifications = {"tasking": {"order_status": {"email": True}}}

        response = client.id_v2.edit_user_settings(notifications)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry
        (
            pact.upon_receiving("Request to edit the user settings")
            .given("user exists")
            .with_request(method="PUT", path="/user/settings")
            .with_body(
                {
                    "notifications": [
                        {
                            "category": "tasking",
                            "settings": [
                                {"email": False, "topic": "tasking:order_status"}
                            ],
                        }
                    ]
                }
            )
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Edit user settings with invalid payload")
    @description("Edit the user settings with an invalid payload.")
    def test_edit_user_settings_invalid_notifications_payload(
        self, oauth_token_entry, client, pact
    ):
        api_path = "id/v2/user/settings"
        id_response_body = json.dumps(
            {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}
        )
        Entry.single_register(
            "PUT",
            client._gateway_url + api_path,
            status=422,
            body=json.dumps(
                {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}
            ),
        )

        with raises(
            IDAPIError,
            match=re.escape(f"ID API Error - 422 : {id_response_body}"),
        ):
            client.id_v2.edit_user_settings(
                {"tasking": {"order_status": {"email": "wrong"}}}
            )

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Invalid request to edit the user settings")
            .given("user exists")
            .with_request(method="PUT", path="/user/settings")
            .with_body(
                {
                    "notifications": [
                        {
                            "category": "tasking",
                            "settings": [
                                {"email": "False", "topic": "tasking:order_status"}
                            ],
                        }
                    ]
                }
            )
            .will_respond_with(422)
            .with_body(id_response_body)
        )

    @title("Get credit balance")
    @description("Retrieve the credit balance for a contract.")
    def test_get_credit_balance(self, oauth_token_entry, client, pact):
        contract_id = str(uuid4())
        api_path = f"id/v2/{contract_id}/wallet/credit"
        id_response_body = json.dumps(
            {
                "currency": "GBP",
                "balance": 100000,
            }
        )

        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=200,
            body=id_response_body,
        )

        response = client.id_v2.get_credit_balance(contract_id)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Request for the contract's balance")
            .given("contract exists")
            .with_request(method="GET", path=f"/{contract_id}/wallet/credit")
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Create webhook")
    @description("Create a webhook with a given payload.")
    def test_create_webhook(self, oauth_token_entry, client, pact):
        api_path = "id/v2/webhooks/"

        name = "My Webhook"
        url = "https://my-webhook-url.com"
        id_response_body = json.dumps(
            {
                "active": True,
                "event_types": [
                    {
                        "topic": "tasking:order_status",
                    }
                ],
                "name": name,
                "url": url,
                "id": str(uuid4()),
                "signing_key": "string",
            }
        )

        Entry.single_register(
            "POST", client._gateway_url + api_path, status=200, body=id_response_body
        )

        response = client.id_v2.create_webhook(name, url, ["tasking:order_status"])
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Create a webhook")
            .given("user exists")
            .with_request(method="POST", path="/webhooks")
            .with_body(
                {
                    "event_types": ["tasking:order_status"],
                    "name": "My Webhook 2",
                    "url": "https://my-webhook-url-2.com",
                }
            )
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Create webhook with invalid payload")
    @description("Create a webhook with an invalid payload.")
    def test_create_webhook_invalid_payload(self, oauth_token_entry, client, pact):
        api_path = "id/v2/webhooks/"

        id_response_body = json.dumps(
            {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}
        )

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            status=422,
            body=id_response_body,
        )

        with raises(
            IDAPIError,
            match=re.escape(
                'ID API Error - 422 : {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}'
            ),
        ):
            client.id_v2.create_webhook(
                "My Webhook", "https://my-webhook-url.com", ["tasking:order_status"]
            )

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Create a webhook invalid payload")
            .given("user exists")
            .with_request(method="POST", path="/webhooks")
            .will_respond_with(422)
            .with_body(id_response_body)
        )

    @title("Get webhook")
    @description("Retrieve the details of a webhook.")
    def test_get_webhook(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/"
        id_response_body = json.dumps(
            {
                "active": True,
                "event_types": [
                    {
                        # "description": "Receive notifications for all tasking order updates", this isn't in the response at the moment.
                        # "name": "Tasking order status updates", neither is this one.
                        "topic": "tasking:order_status",
                    }
                ],
                "name": "My Webhook",
                "url": "https://my-webhook-url.com",
                "id": webhook_id,
            }
        )

        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=200,
            body=id_response_body,
        )

        response = client.id_v2.get_webhook(webhook_id)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Get webhook information")
            .given("webhook exists")
            .with_request(method="GET", path=f"/webhooks/{webhook_id}")
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Get webhook (not found)")
    @description("Retrieve the details of a webhook that does not exist.")
    def test_get_webhook_not_found(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/"
        id_response_body = json.dumps(
            {"detail": f"Webhook with ID {webhook_id} not found."}
        )
        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=404,
            body=id_response_body,
        )

        with raises(IDAPIError):
            client.id_v2.get_webhook(webhook_id)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Webhook not found")
            .given("webhook does not exist")
            .with_request(method="GET", path=f"/webhooks/{webhook_id}")
            .will_respond_with(404)
            .with_body(id_response_body)
        )

    @title("List webhooks")
    @description("Retrieve a list of webhooks.")
    def test_list_webhooks(self, oauth_token_entry, client, pact):
        token = "token"
        api_path = f"id/v2/webhooks/?per_page=25&token={token}"
        id_response_body = json.dumps(
            {
                "webhooks": [
                    {
                        "active": True,
                        "event_types": [
                            {
                                "topic": "tasking:order_status",
                            }
                        ],
                        "name": "My Webhook",
                        "url": "https://my-webhook-url.com",
                        "id": "532fcb53-4d79-4c4e-98b7-4c1a92290b06",
                    }
                ],
                "context": {"per_page": 25, "matched": 10, "returned": 10},
                "links": [{"href": "http://example.com", "rel": "next"}],
            }
        )

        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=200,
            body=id_response_body,
        )

        response = client.id_v2.list_webhooks(page_token=token)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("List webhooks")
            .given("webhooks exist")
            .with_request(method="GET", path="/webhooks")
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("List webhooks (invalid token)")
    @description("Retrieve a list of webhooks with an invalid token.")
    def test_list_webhooks_invalid_token(self, oauth_token_entry, client, pact):
        token = "invalid-token"
        api_path = f"id/v2/webhooks/?per_page=25&token={token}"
        id_response_body = json.dumps(
            {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}
        )
        Entry.single_register(
            "GET", client._gateway_url + api_path, status=422, body=id_response_body
        )

        with raises(
            IDAPIError,
            match=re.escape(
                'ID API Error - 422 : {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}'
            ),
        ):
            client.id_v2.list_webhooks(page_token=token)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Get webhook with invalid token")
            .given("webhooks exist")
            .with_request(method="GET", path="/webhooks")
            .with_query_parameters({"per_page": "25", "token": token})
            .will_respond_with(422)
            .with_body(id_response_body)
        )

    @title("Edit webhook")
    @description("Edit a webhook with a given payload.")
    def test_edit_webhook(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/"

        name = "My Webhook"
        url = "https://my-webhook-url.com"
        id_response_body = json.dumps(
            {
                "active": True,
                "event_types": [
                    {
                        "topic": "tasking:order_status",
                    }
                ],
                "name": name,
                "url": url,
                "id": webhook_id,
            }
        )

        Entry.single_register(
            "PATCH", client._gateway_url + api_path, status=200, body=id_response_body
        )
        response = client.id_v2.edit_webhook(webhook_id, active=False, name=name)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Edit webhook")
            .given("webhook exists")
            .with_request(method="PATCH", path=f"/webhooks/{webhook_id}")
            .with_body(
                json.dumps(
                    {
                        "active": True,
                        "event_types": ["tasking:order_status"],
                        "name": "My Webhook",
                    }
                )
            )
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Edit webhook with invalid payload")
    @description("Edit a webhook with an invalid payload.")
    def test_edit_webhook_invalid_payload(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/"
        id_response_body = json.dumps(
            {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}
        )

        Entry.single_register(
            "PATCH",
            client._gateway_url + api_path,
            status=422,
            body=id_response_body,
        )

        with raises(
            IDAPIError,
            match=re.escape(
                'ID API Error - 422 : {"detail": [{"loc": ["string"], "msg": "string", "type": "string"}]}'
            ),
        ):
            client.id_v2.edit_webhook(webhook_id, active=False, event_types=["unknown"])

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Edit webhook with invalid payload")
            .given("webhook payload is invalid")
            .with_request(method="PATCH", path=f"/webhooks/{webhook_id}")
            .with_body(
                {
                    "active": True,
                    "event_types": ["tasking:order_status"],
                    "name": "My Webhook",
                }
            )
            .will_respond_with(422)
            .with_body(id_response_body)
        )

    @title("Delete webhook")
    @description("Delete a webhook")
    def test_delete_webhook(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/"

        Entry.single_register(
            "DELETE",
            client._gateway_url + api_path,
            status=204,
        )

        response = client.id_v2.delete_webhook(webhook_id)
        assert not response

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Delete webhook")
            .given("webhook exists")
            .with_request(method="DELETE", path=f"/webhooks/{webhook_id}")
            .will_respond_with(204)
        )

    @title("Delete webhook (not found)")
    @description("Delete a webhook that does not exist.")
    def test_delete_webhook_not_found(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/"
        id_response_body = json.dumps(
            {"detail": f"Webhook with ID {webhook_id} not found."}
        )

        Entry.single_register(
            "DELETE",
            client._gateway_url + api_path,
            status=404,
        )

        with raises(IDAPIError):
            client.id_v2.delete_webhook(webhook_id)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Attempt to delete a nonexistent webhook")
            .given("webhook does not exist")
            .with_request(method="DELETE", path=f"/webhooks/{webhook_id}")
            .will_respond_with(404)
            .with_body(id_response_body)
        )

    @title("Rotate webhook signing key")
    @description("Rotate the webhook signing key.")
    def test_rotate_webhook_signing_key(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/rotate/"
        id_response_body = json.dumps(
            {
                "active": True,
                "event_types": [
                    {
                        "topic": "tasking:order_status",
                    }
                ],
                "name": "My Webhook",
                "url": "https://my-webhook-url.com",
                "id": webhook_id,
                "signing_key": "string",
            }
        )

        Entry.single_register(
            "POST", client._gateway_url + api_path, status=200, body=id_response_body
        )

        response = client.id_v2.rotate_webhook_signing_key(webhook_id)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Rotate webhook signing key")
            .given("webhook does not exist")
            .with_request(method="POST", path=f"/webhooks/{webhook_id}/rotate")
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Rotate webhook signing key (not found)")
    @description("Rotate the signing key of a webhook that does not exist.")
    def test_rotate_webhook_signing_key_not_found(
        self, oauth_token_entry, client, pact
    ):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/rotate/"
        id_response_body = json.dumps(
            {"detail": f"Webhook with ID {webhook_id} not found."}
        )
        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            status=404,
        )

        with raises(IDAPIError):
            client.id_v2.rotate_webhook_signing_key(webhook_id)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Rotate webhook signing key not found")
            .given("webhook does not exist")
            .with_request(method="POST", path=f"/webhooks/{webhook_id}/rotate")
            .will_respond_with(404)
            .with_body(id_response_body)
        )

    @title("Test webhook")
    @description("Test an existing webhook.")
    def test_test_webhook(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/test/"
        id_response_body = json.dumps(
            {
                "active": True,
                "event_types": [
                    {
                        "topic": "tasking:order_status",
                    }
                ],
                "name": "My Webhook",
                "url": "https://my-webhook-url.com",
                "id": webhook_id,
                "webhook_result": {
                    "success": True,
                    "status_code": 200,
                    "detail": "string",
                },
            }
        )

        Entry.single_register(
            "POST", client._gateway_url + api_path, status=200, body=id_response_body
        )

        response = client.id_v2.test_webhook(webhook_id)
        assert isinstance(response, dict)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Test webhook")
            .given("webhook exists")
            .with_request(method="POST", path=f"/webhooks/{webhook_id}/test")
            .will_respond_with(200)
            .with_body(id_response_body)
        )

    @title("Test webhook (not found)")
    @description("Test a webhook that does not exist.")
    def test_test_webhook_not_found(self, oauth_token_entry, client, pact):
        webhook_id = str(uuid4())
        api_path = f"id/v2/webhooks/{webhook_id}/test/"

        id_response_body = json.dumps(
            {"detail": f"Webhook with ID {webhook_id} not found."}
        )

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            status=404,
        )

        with raises(IDAPIError):
            client.id_v2.test_webhook(webhook_id)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Test webhook not found")
            .given("webhook does not exist")
            .with_request(method="POST", path=f"/webhooks/{webhook_id}/test")
            .will_respond_with(404)
            .with_body(id_response_body)
        )

    @title("Get webhook events")
    @description("Retrieve a list of webhook events.")
    def test_get_webhook_events(self, oauth_token_entry, client, pact):
        api_path = "id/v2/webhooks/events/"
        id_response_body = json.dumps(
            [
                {
                    "topic": "tasking:order_status",
                    "name": "string",
                    "description": "string",
                }
            ]
        )
        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            status=200,
            body=id_response_body,
        )

        response = client.id_v2.get_webhook_events()
        assert isinstance(response, list)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == "/" + api_path
        assert api_request.headers["authorization"] == oauth_token_entry

        (
            pact.upon_receiving("Get webhook events")
            .given("webhooks exist")
            .with_request(method="GET", path="/webhooks/events/")
            .will_respond_with(200)
            .with_body(id_response_body)
        )



from allure import description, story, title
import tempfile
from io import BytesIO
from json import dumps
from unittest.mock import patch
from urllib.parse import urljoin, urlparse
from uuid import uuid4

import pytest
from mocket import Mocket
from mocket.mockhttp import Entry, Response
from pytest import mark

from satellitevu.apis.orders import bytes_to_file
from satellitevu.auth.exc import Api401Error, Api403Error

API_PATH = "orders/v2/contract-id/"


@mark.usefixtures("mocketize_fixture")
@story("Orders")
class TestOrders:
    @title("Submit order (single item)")
    @description("Submit an order for a single item")
    @mark.parametrize("pact", ["cos"], indirect=True)
    @mark.parametrize(
        "item_ids",
        (
            ("20221005T214049000_basic_0_TABI"),
            ("20220923T222227000_basic_0_TABI"),
        ),
    )
    def test_submit_single_item(self, client, oauth_token_entry, item_ids, pact):
        payload = dumps({"item_id": [item_ids]})
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            body=dumps(payload),
            status=201,
        )

        response = client.orders_v2.submit(contract_id=contract_id, item_ids=item_ids)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}"
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry
        assert api_request.body == payload

        assert response.status == 201
        cos_response_body = dumps(
            {
                "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
                        "geometry": {"type": "Point", "coordinates": [0, 0]},
                        "properties": {
                            "item_id": "20220923T222227000_basic_0_TABI",
                            "order_id": "1cfdc8cc-968d-406c-bada-d45c55a26857",
                            "created_at": "2019-08-24T14:15:22Z",
                            "stac_metadata": {
                                "id": "string",
                                "collection": "string",
                                "assets": {
                                    "property1": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                    "property2": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                },
                                "bbox": [0, 0, 0, 0],
                                "properties": {
                                    "eo:cloud_cover": 0,
                                    "datetime": "2019-08-24T14:15:22Z",
                                    "gsd": 0,
                                    "platform": "string",
                                    "view:azimuth": 0,
                                    "view:off_nadir": 0,
                                    "view:sun_azimuth": 0,
                                    "view:sun_elevation": 0,
                                },
                            },
                            "price": {"value": 100000, "currency": "GBP"},
                        },
                    }
                ],
                "owned_by": "John Doe",
                "created_at": "2019-08-24T14:15:22Z",
                "updated_at": "2019-08-24T14:15:22Z",
                "contract_id": "9aafc1a8-e497-46c9-ba0b-bd5b03c353e4",
                "price": {"value": 100000, "currency": "GBP"},
            }
        )

        (
            pact.upon_receiving("An order for a single item")
            .given("contract has credit")
            .with_request(method="POST", path=f"/{contract_id}/")
            .with_body(payload)
            .will_respond_with(201)
            .with_body(cos_response_body)
        )

    @title("Submit order (multiple items)")
    @description("Submit an order for a single item")
    @mark.parametrize("pact", ["cos"], indirect=True)
    @mark.parametrize(
        "item_ids",
        (
            (["20221005T214049000_basic_0_TABI", "20220923T222227000_basic_0_TABI"]),
            (["20220923T222227000_basic_0_TABI"]),
        ),
    )
    def test_submit_multiple_items(self, client, oauth_token_entry, item_ids, pact):
        payload = dumps({"item_id": item_ids})
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        Entry.single_register(
            "POST",
            client._gateway_url + api_path,
            body=dumps(payload),
            status=201,
        )

        response = client.orders_v2.submit(contract_id=contract_id, item_ids=item_ids)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}"
        assert api_request.headers["content-type"] == "application/json"
        assert api_request.headers["authorization"] == oauth_token_entry
        assert api_request.body == payload

        assert response.status == 201

        cos_response_body = dumps(
            {
                "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
                "type": "FeatureCollection",
                "features": [
                    {
                        "type": "Feature",
                        "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
                        "geometry": {"type": "Point", "coordinates": [0, 0]},
                        "properties": {
                            "item_id": "20221005T214049000_basic_0_TABI",
                            "order_id": "1cfdc8cc-968d-406c-bada-d45c55a26857",
                            "created_at": "2019-08-24T14:15:22Z",
                            "stac_metadata": {
                                "id": "string",
                                "collection": "string",
                                "assets": {
                                    "property1": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                    "property2": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                },
                                "bbox": [0, 0, 0, 0],
                                "properties": {
                                    "eo:cloud_cover": 0,
                                    "datetime": "2019-08-24T14:15:22Z",
                                    "gsd": 0,
                                    "platform": "string",
                                    "view:azimuth": 0,
                                    "view:off_nadir": 0,
                                    "view:sun_azimuth": 0,
                                    "view:sun_elevation": 0,
                                },
                            },
                            "price": {"value": 100000, "currency": "GBP"},
                        },
                    },
                    {
                        "type": "Feature",
                        "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
                        "geometry": {"type": "Point", "coordinates": [0, 0]},
                        "properties": {
                            "item_id": "20220923T222227000_basic_0_TABI",
                            "order_id": "1cfdc8cc-968d-406c-bada-d45c55a26857",
                            "created_at": "2019-08-24T14:15:22Z",
                            "stac_metadata": {
                                "id": "string",
                                "collection": "string",
                                "assets": {
                                    "property1": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                    "property2": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                },
                                "bbox": [0, 0, 0, 0],
                                "properties": {
                                    "eo:cloud_cover": 0,
                                    "datetime": "2019-08-24T14:15:22Z",
                                    "gsd": 0,
                                    "platform": "string",
                                    "view:azimuth": 0,
                                    "view:off_nadir": 0,
                                    "view:sun_azimuth": 0,
                                    "view:sun_elevation": 0,
                                },
                            },
                            "price": {"value": 100000, "currency": "GBP"},
                        },
                    },
                    {
                        "type": "Feature",
                        "id": "497f6eca-6276-4993-bfeb-53cbbbba6f08",
                        "geometry": {"type": "Point", "coordinates": [0, 0]},
                        "properties": {
                            "item_id": "20220923T222227000_basic_0_TABI",
                            "order_id": "1cfdc8cc-968d-406c-bada-d45c55a26857",
                            "created_at": "2019-08-24T14:15:22Z",
                            "stac_metadata": {
                                "id": "string",
                                "collection": "string",
                                "assets": {
                                    "property1": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                    "property2": {
                                        "href": "http://example.com",
                                        "type": "string",
                                        "roles": ["string"],
                                    },
                                },
                                "bbox": [0, 0, 0, 0],
                                "properties": {
                                    "eo:cloud_cover": 0,
                                    "datetime": "2019-08-24T14:15:22Z",
                                    "gsd": 0,
                                    "platform": "string",
                                    "view:azimuth": 0,
                                    "view:off_nadir": 0,
                                    "view:sun_azimuth": 0,
                                    "view:sun_elevation": 0,
                                },
                            },
                            "price": {"value": 100000, "currency": "GBP"},
                        },
                    },
                ],
                "owned_by": "John Doe",
                "created_at": "2019-08-24T14:15:22Z",
                "updated_at": "2019-08-24T14:15:22Z",
                "contract_id": "9aafc1a8-e497-46c9-ba0b-bd5b03c353e4",
                "price": {"value": 100000, "currency": "GBP"},
            }
        )

        (
            pact.upon_receiving("An order for multiple items")
            .given("contract has credit")
            .with_request(method="POST", path=f"/{contract_id}/")
            .with_body(payload)
            .will_respond_with(201)
            .with_body(cos_response_body)
        )

    @title("Get order download URL")
    @description("Download order URL")
    def test_item_download_url(self, client, oauth_token_entry, redirect_response):
        order_id = str(uuid4())
        item_id = "20221005T214049000_basic_0_TABI"
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        Entry.single_register(
            "GET",
            client._gateway_url
            + f"{api_path}{order_id}/{item_id}/download?redirect=False",
            body=dumps(redirect_response),
        )

        response = client.orders_v2.item_download_url(
            contract_id=contract_id, order_id=order_id, item_id=item_id
        )

        requests = Mocket.request_list()

        assert len(requests) == 2
        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert (
            api_request.path
            == f"/{api_path}{order_id}/{item_id}/download?redirect=False"
        )
        assert api_request.headers["authorization"] == oauth_token_entry

        assert isinstance(response, dict)
        assert response["url"] == "https://image.test"

    @title("Download an order (unauthorized")
    @description("Attempt to download an order without authorization")
    @mark.parametrize(
        ["status", "exception"],
        (
            (401, Api401Error),
            (403, Api403Error),
        ),
    )
    def test_no_access_to_download_if_unauthorized(
        self, client, oauth_token_entry, status, exception, redirect_response
    ):
        order_id = str(uuid4())
        item_id = "20231110T173102000_visual_30_hotsat"
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        Entry.single_register(
            "GET",
            client._gateway_url
            + f"{api_path}{order_id}/{item_id}/download?redirect=False",
            body=dumps(redirect_response),
            status=status,
        )

        with pytest.raises(exception):
            client.orders_v2.item_download_url(
                contract_id=contract_id, order_id=order_id, item_id=item_id
            )

        requests = Mocket.request_list()

        assert len(requests) == 2
        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert (
            api_request.path
            == f"/{api_path}{order_id}/{item_id}/download?redirect=False"
        )
        assert api_request.headers["authorization"] == oauth_token_entry

    @title("Download order item")
    @description("Download an order")
    @mark.parametrize("pact", ["cos"], indirect=True)
    def test_download_order_item(
        self, client, oauth_token_entry, redirect_response, pact
    ):
        order_id = str(uuid4())
        item_id = "20231110T173102000_visual_30_hotsat"
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))
        download_dir = "downloads"

        Entry.register(
            "GET",
            client._gateway_url
            + f"{api_path}{order_id}/{item_id}/download?redirect=False",
            Response(headers={"Retry-After": "1"}, status=202),
            Response(body=dumps(redirect_response), status=200),
        )
        Entry.single_register("GET", uri=redirect_response["url"])

        with patch("satellitevu.apis.orders.bytes_to_file") as mock_file_dl:
            mock_file_dl.return_value = f"{download_dir}/image.zip"

            response = client.orders_v2.download_item(
                contract_id=contract_id,
                order_id=order_id,
                item_id=item_id,
                destdir=download_dir,
            )

        requests = Mocket.request_list()

        assert len(requests) == 4

        api_request = requests[2]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert (
            api_request.path
            == f"/{api_path}{order_id}/{item_id}/download?redirect=False"
        )
        assert api_request.headers["authorization"] == oauth_token_entry

        mock_file_dl.assert_called_once()
        assert response == mock_file_dl()
        assert isinstance(response, str)

        Mocket.assert_fail_if_entries_not_served()

        (
            pact.upon_receiving("A Request to download an order item")
            .given("contract has credit")
            .with_request(
                method="GET", path=f"/{contract_id}/{order_id}/{item_id}/download"
            )
            .with_query_parameter("redirect", "false")
            .will_respond_with(200)
            .with_body(redirect_response)
        )

    @title("Get order details")
    @description("Get the details of an existing order")
    @mark.parametrize("pact", ["cos"], indirect=True)
    def test_get_order_details(
        self, client, oauth_token_entry, order_details_response, pact
    ):
        contract_id = str(uuid4())
        order_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        Entry.single_register(
            "GET",
            client._gateway_url + f"{api_path}{order_id}",
            body=dumps(order_details_response),
        )

        response = client.orders_v2.get_order_details(
            contract_id=contract_id, order_id=order_id
        )

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}{order_id}"
        assert api_request.headers["authorization"] == oauth_token_entry

        assert isinstance(response, dict)

        (
            pact.upon_receiving("Details for an existing order")
            .given("order exists")
            .with_request(method="GET", path=f"/{contract_id}/{order_id}/")
            .will_respond_with(200)
            .with_body(order_details_response)
        )

    @title("Get orders")
    @description("Get a list of orders")
    @mark.parametrize("pact", ["cos"], indirect=True)
    def test_get_orders(self, client, oauth_token_entry, order_list_response, pact):
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        Entry.single_register(
            "GET",
            client._gateway_url + api_path,
            body=dumps(order_list_response),
        )

        response = client.orders_v2.get_orders(contract_id=contract_id)

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}"
        assert api_request.headers["authorization"] == oauth_token_entry

        assert isinstance(response, dict)

        (
            pact.upon_receiving("Get list of orders")
            .given("order exists")
            .with_request(method="GET", path=f"/{contract_id}/")
            .will_respond_with(200)
            .with_body(order_list_response)
        )

    @title("Get order details (unauthorized)")
    @description("Attempt to get order details without authorization")
    @mark.parametrize(
        ["status", "exception"],
        (
            (401, Api401Error),
            (403, Api403Error),
        ),
    )
    def test_cannot_get_order_details_if_unauthorized(
        self, client, oauth_token_entry, status, exception, order_details_response
    ):
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        order_id = str(uuid4())

        Entry.single_register(
            "GET",
            client._gateway_url + f"{api_path}{order_id}",
            body=dumps(order_details_response),
            status=status,
        )

        with pytest.raises(exception):
            client.orders_v2.get_order_details(
                contract_id=contract_id, order_id=order_id
            )

        requests = Mocket.request_list()
        assert len(requests) == 2

        api_request = requests[-1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}{order_id}"
        assert api_request.headers["authorization"] == oauth_token_entry

    @title("Download order")
    @description("Download an order")
    def test_download_order(
        self,
        client,
        oauth_token_entry,
        redirect_response,
    ):
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))
        order_id = "528b0f77-5df1-4ed7-9224-502817170613"
        download_dir = "downloads"

        Entry.register(
            "GET",
            client._gateway_url + f"{api_path}{order_id}/download?redirect=False",
            Response(headers={"Retry-After": "1"}, status=202),
            Response(body=dumps(redirect_response), status=200),
        )
        Entry.single_register("GET", uri=redirect_response["url"])

        with patch("satellitevu.apis.orders.bytes_to_file") as mock_file_dl:
            mock_file_dl.return_value = f"{download_dir}/{order_id}.zip"

            response = client.orders_v2.download_order(
                contract_id=contract_id, order_id=order_id, destdir=download_dir
            )

        requests = Mocket.request_list()

        assert len(requests) == 4

        api_request = requests[1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}{order_id}/download?redirect=False"
        assert api_request.headers["authorization"] == oauth_token_entry

        mock_file_dl.assert_called_once()
        assert response == mock_file_dl()
        assert isinstance(response, str)

        Mocket.assert_fail_if_entries_not_served()

    @mark.parametrize(
        ["status", "exception"],
        (
            (401, Api401Error),
            (403, Api403Error),
        ),
    )
    @title("Download an order (unauthorized)")
    @description("Attempt to download an order without authorization")
    def test_download_order_unauthorized(
        self,
        client,
        redirect_response,
        status,
        exception,
    ):
        contract_id = str(uuid4())
        api_path = API_PATH.replace("contract-id", str(contract_id))

        order_id = "528b0f77-5df1-4ed7-9224-502817170613"
        download_dir = "downloads"

        Entry.single_register(
            "POST",
            urljoin(client.auth.auth_url, "oauth/token"),
            body=dumps({"access_token": None}),
        )

        Entry.single_register(
            "GET",
            client._gateway_url + f"{api_path}{order_id}/download?redirect=False",
            body=dumps(redirect_response),
            status=status,
        )

        with pytest.raises(exception):
            client.orders_v2.download_order(
                contract_id=contract_id, order_id=order_id, destdir=download_dir
            )

        requests = Mocket.request_list()

        assert len(requests) == 2

        api_request = requests[1]
        assert api_request.headers["host"] == urlparse(client._gateway_url).hostname
        assert api_request.path == f"/{api_path}{order_id}/download?redirect=False"
        assert api_request.headers["authorization"] == "Bearer None"

    @title("Write bytes to file")
    @description("Write bytes to a file")
    def test_bytes_to_file(self):
        outfile_path = tempfile.mkstemp()[1]
        output = bytes_to_file(BytesIO(b"Hello world"), outfile_path)

        assert isinstance(output, str)
